//
//  YHFileListItems.swift
//  galaxy
//
//  Created by davidhuangA on 2024/3/21.
//  Copyright © 2024 https://www.galaxy-immi.com. All rights reserved.
//

import UIKit

class YHFileListItemsView: UIView {
    
    var orderId:Int = 0
    var viewModel: YHCertificateViewModel = YHCertificateViewModel()
  
    override init(frame: CGRect) {
        super.init(frame: frame)
    }
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    var dataModel : YHFileListModel? {
        didSet {
            updateUI()
        }
    }
    
    
    private lazy var topBkg : UIView = {
        let imagV = UIImageView()
        imagV.image = UIImage(named: "file_list_session_bkg")
        imagV.contentMode = .scaleAspectFill
        return imagV
    }()
    
    
    
    private lazy var titleLable : UILabel = {
        let lable0 = UILabel(text: "title")
        lable0.textColor = UIColor.mainTextColor
        lable0.font = UIFont.PFSC_R(ofSize: 18)
        lable0.textAlignment = .center
        return lable0
    }()
    
    
    
    
    
}

private extension YHFileListItemsView {
    func updateUI() {
        removeSubviews()
        guard let dataModel = dataModel else { return }
        addSubview(topBkg)
        topBkg.snp.makeConstraints { make in
            make.top.equalTo(22)
            make.centerX.equalToSuperview()
            make.width.equalTo(146)
            make.height.equalTo(28)
        }
        
        titleLable.text = dataModel.ApplicantName.defaultStringIfEmpty(detaultValue: "--")
        addSubview(titleLable)
        titleLable.snp.makeConstraints { make in
            make.top.equalTo(22)
            make.centerX.equalToSuperview()
            make.left.right.equalToSuperview()
            make.height.equalTo(28)
        }
        
        
        let holdView : UIView = UIView()
        holdView.layer.cornerRadius = kCornerRadius6
        holdView.clipsToBounds = true
        holdView.layer.borderWidth = 0.5
        holdView.layer.borderColor = UIColor.separatorColor.cgColor
        
        addSubview(holdView)
        holdView.snp.makeConstraints { make in
            make.top.equalTo(titleLable.snp.bottom).offset(14)
            make.left.equalToSuperview().offset(kMargin)
            make.right.equalToSuperview().offset(-kMargin)
            make.bottom.equalToSuperview()
        }
        
        let sessionView : UIView = UIView()
        sessionView.backgroundColor = UIColor(hex: 0xD9E4FD, alpha: 1.0)
        holdView.addSubview(sessionView)
        
        if dataModel.supplement_list.isEmpty {
            sessionView.snp.makeConstraints { make in
                make.top.left.right.equalToSuperview()
                make.height.equalTo(38)
                
                make.bottom.equalToSuperview()
            }
        } else {
            sessionView.snp.makeConstraints { make in
                make.top.left.right.equalToSuperview()
                make.height.equalTo(38)
            }
        }
        
        
        let leftLabel : UILabel = UILabel()
        leftLabel.text = "内容"
        leftLabel.font = UIFont.PFSC_R(ofSize: 12)
        leftLabel.textColor = UIColor.labelTextColor2
        
        holdView.addSubview(leftLabel)
        leftLabel.snp.makeConstraints { make in
            make.left.equalTo(40)
            make.top.equalToSuperview()
            make.height.equalTo(38)
        }
        
        
        let rightLabel : UILabel = UILabel()
        rightLabel.text = "状态"
        rightLabel.font = UIFont.PFSC_R(ofSize: 12)
        rightLabel.textColor = UIColor.labelTextColor2
        
        holdView.addSubview(rightLabel)
        rightLabel.snp.makeConstraints { make in
            make.right.equalTo(-34)
            make.top.equalToSuperview()
            make.height.equalTo(38)
        }
        
        
        var lastView : UIView = sessionView
        for (index,item) in dataModel.supplement_list.enumerated() {
            
            let view = YHFileListItemView()
            view.myIndex = index
            view.dataModel = item
            view.clickBlock = {
                [weak self] model in
                guard let self = self else { return }
                self.handleClickDataModel(model)
            }
            holdView.addSubview(view)
            
            
            if index == dataModel.supplement_list.count - 1 {
                view.snp.makeConstraints { make in
                    make.top.equalTo(lastView.snp.bottom)
                    make.left.right.equalToSuperview()
                    make.bottom.equalToSuperview()
                }
            } else {
                view.snp.makeConstraints { make in
                    make.top.equalTo(lastView.snp.bottom)
                    make.left.right.equalToSuperview()
                }
            }
            lastView = view
        }
        
        holdView.layer.cornerRadius = kCornerRadius6
        holdView.clipsToBounds = true
        holdView.layer.borderWidth = 1
        holdView.layer.borderColor = UIColor(hex: 0xDEE5F1, alpha: 1.0).cgColor
    }
}

private extension YHFileListItemsView {
    
    func handleClickDataModel(_ targetModel: YHSupplementInfo?) {
        
        guard let model = targetModel else {
            return
        }
        
        if model.checkStatus == YHCertificateUploadStatus.preUpload.rawValue { // 待上传
            showUploadSheetView(model)
            return
        }
        // 非待上传的进入详情
        let vc = YHCertificateUploadContentListVC()
        vc.orderId = self.orderId
        vc.categoryId = self.dataModel?.category_id ?? 0
        vc.supplementId = model.id
        UIViewController.current?.navigationController?.pushViewController(vc)
    }
    
    
    func showUploadSheetView(_ supplementInfo: YHSupplementInfo) {
        
        let sheetView = YHCertificateUploadSheetView.sheetView()
        sheetView.maxSelectCount = 9
         sheetView.uploadFilesBlock = {
            [weak self] fileUrl in
            guard let self = self else { return }
            print(fileUrl)
             uploadFile(fileUrl, supplementInfo: supplementInfo) {
                 // 待上传的进入详情
                 DispatchQueue.main.async {
                     let vc = YHCertificateUploadContentListVC()
                     vc.orderId = self.orderId
                     vc.categoryId = self.dataModel?.category_id ?? 0
                     vc.supplementId = supplementInfo.id
                     UIViewController.current?.navigationController?.pushViewController(vc)
                 }
             }
        }
        sheetView.uploadImageBlock = {
            [weak self] targetImgArr in
            guard let self = self else { return }
            let ossGroup = DispatchGroup()
            var ossArr:[YHContentUrl] = []

            YHHUD.show(.progress(message: "上传中..."))
            // 先将所有图片组上传到OSS
            for item in targetImgArr {
                if let image = item.data {
                    ossGroup.enter()
                    uploadImage(image, imageName:item.name, supplementInfo: supplementInfo) {
                        success, urlModel in
                        if success, let model = urlModel {
                            ossArr.append(model)
                        }
                        ossGroup.leave()
                    }
                }
            }
            
            // 再将OSS上传信息组上传到服务器
            ossGroup.notify(queue: .main) {
                if ossArr.count <= 0 {
                    YHHUD.hide()
                    YHHUD.flash(message: "上传失败")
                    return
                }
                
                self.viewModel.updateCertificateItem(supplementsId:supplementInfo.id, urlModels:ossArr, operation: "add", rename: "") {
                    [weak self] success, error in
                    guard let self = self else { return }

                    YHHUD.hide()
                    if  success {
                       
                        if ossArr.count == targetImgArr.count {
                            YHHUD.flash(message: "上传成功")
                        } else {
                            YHHUD.flash(message: "\(ossArr.count)张照片上传成功\n\(targetImgArr.count-ossArr.count)张照片上传失败")
                        }
                        let vc = YHCertificateUploadContentListVC()
                        vc.orderId = self.orderId
                        vc.categoryId = self.dataModel?.category_id ?? 0
                        vc.supplementId = supplementInfo.id
                        UIViewController.current?.navigationController?.pushViewController(vc)

                    } else {
                        YHHUD.flash(message: "上传失败")
                    }
                }
            }
        }
        sheetView.show()
    }
    
    func uploadImage(_ img: UIImage, imageName:String, supplementInfo:YHSupplementInfo, callBack:((Bool, YHContentUrl?)->())?)  {
        
        // 先OSS上传得到URL
        self.viewModel.uploadImage(img) {
            url, error in
            if let url = url, !url.isEmpty {
                var suffix = "jpg"
                let suffixName = url.pathExtension.lowercased()
                if !suffixName.isEmpty {
                    suffix = suffixName
                }
                
                // 再调用业务接口
                let model = YHContentUrl()
                model.name = imageName
                model.url = url
                model.suffix = suffix
                callBack?(true, model)
            } else {
                callBack?(false, nil)
            }
        }
    }
    
    func uploadFile(_ fileUrl:URL, supplementInfo:YHSupplementInfo, successBlock:(()->())?)  {
        if let fileData = try? Data(contentsOf: fileUrl) {
            // 将文件数据上传到服务器
            let size = String(format: "%.2fM", Double(fileData.count)/(1024.0 * 1024.0))
            print("\(size)")
            // 先OSS上传得到URL
            YHHUD.show(.progress(message: "上传中..."))
            self.viewModel.uploadFile(fileUrl.absoluteString) {
                [weak self] successUrl, error in
                guard let self = self else { return }
                
                YHHUD.hide()
                
                // 再调用业务接口
                if let successUrl = successUrl, !successUrl.isEmpty {
                    let fileName = successUrl.lastPathComponent
                    let suffixName = successUrl.pathExtension.lowercased()
                    let model = YHContentUrl()
                    model.name = fileName
                    model.url = successUrl
                    model.suffix = suffixName
                    YHHUD.show(.progress(message: "上传中..."))
                    self.viewModel.updateCertificateItem(supplementsId:supplementInfo.id, urlModels:[model], operation: "add", rename: "") {
                        success, error in
                        YHHUD.hide()
//                        self.requestList(status:self.status)
                        if success {
                            YHHUD.flash(message: "上传成功")
                        } else {
                            var msg = "上传失败"
                            if let errorMsg = error?.errorMsg, !errorMsg.isEmpty {
                                msg = errorMsg
                            }
                            YHHUD.flash(message:msg)
                        }
                        if success {
                            successBlock?()
                        }
                    }
                    return
                }
                var msg = "上传图片失败"
                if let errorMsg = error?.errorMsg, !errorMsg.isEmpty {
                    msg = errorMsg
                }
                YHHUD.flash(message: msg)

            }
        } else {
            YHHUD.flash(message: "识别文件失败")
        }
    }
}


